package Tires 
  model IdealTire 
    
    extends Chassis.Interfaces.Tire;
    parameter Modelica.SIunits.Radius tire_radius "Tire radius";
  equation 
    assert(motion.weight > 0,
      "Tire hop: IdealTire does not support tire hopping");
    motion.tire_elevation = 0;
    motion.vehicle_position = hub.phi*2*Modelica.Constants.pi;
    motion.acc_force*tire_radius = -hub.tau "Momentum balance";
    annotation (Icon(Text(
          extent=[-100, 70; 100, 30],
          string="%name",
          style(color=45, fillColor=49)), Text(
          extent=[-100, -10; 100, -50],
          style(color=7),
          string="Ideal")));
  end IdealTire;
  
  model PacejkaTire 
    extends Chassis.Interfaces.Tire;
    parameter RoadSurfaces.PacejkaCoefficients slip_data;
    parameter Real scale_factor "Scale factor for mu";
    parameter Modelica.SIunits.Position tire_radius(final min=1e-5) 
      "Wheel radius";
    parameter Real delta=1e-1 "Small speed difference";
  protected 
    Real percent_abs_wheel_slip;
    Real wheel_slip "Slip between wheel and road";
    Modelica.SIunits.Velocity patch_velocity "Theoretical tire patch velocity";
    Real mu "Friction coefficient";
    Modelica.SIunits.AngularVelocity omega "Tire speed";
    Modelica.SIunits.Velocity vehicle_velocity;
    annotation (
      Icon(Text(
          extent=[-100, 70; 100, 30],
          string="%name",
          style(color=45, fillColor=49)), Text(
          extent=[-100, -10; 100, -50],
          string="Mu-Slip",
          style(color=7))),
      Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[2, 2],
        component=[20, 20]),
      Window(
        x=0.33,
        y=0.1,
        width=0.6,
        height=0.6));
  equation 
    assert(motion.weight > 0,
      "Tire hop: IdealTire does not support tire hopping");
    motion.tire_elevation = 0;
    omega = der(hub.phi);
    vehicle_velocity = der(motion.vehicle_position);
    patch_velocity = omega*tire_radius;
    wheel_slip = -(patch_velocity - vehicle_velocity)/(max(patch_velocity,
      vehicle_velocity) + delta);
    percent_abs_wheel_slip = noEvent(abs(wheel_slip)*100);
    mu = scale_factor*slip_data.D*sin(slip_data.C*Modelica.Math.atan(
      slip_data.B*percent_abs_wheel_slip - slip_data.E*(slip_data.B*
      percent_abs_wheel_slip - Modelica.Math.atan(slip_data.B*
      percent_abs_wheel_slip))));
    motion.acc_force = noEvent(motion.weight*mu*sign(wheel_slip));
    hub.tau = -motion.acc_force*tire_radius;
  end PacejkaTire;
  
  package RoadSurfaces 
    final record DryAsphalt 
      extends PacejkaCoefficients(
        B=0.1038,
        C=1.65,
        D=0.9783,
        E=0.65663);
    end DryAsphalt;
    constant DryAsphalt dry_asphalt;
    
    final record Gravel 
      extends PacejkaCoefficients(
        B=0.1207,
        C=0.7,
        D=0.77156,
        E=0.44137);
    end Gravel;
    constant Gravel gravel;
    
    final record Ice 
      extends PacejkaCoefficients(
        B=0.28872,
        C=1.65,
        D=0.55515,
        E=0.64096);
    end Ice;
    constant Ice ice;
    
    final record Snow 
      extends PacejkaCoefficients(
        B=0.14395,
        C=0.9,
        D=0.30434,
        E=-6.439);
    end Snow;
    constant Snow snow;
    
    final record WetAsphalt 
      extends PacejkaCoefficients(
        B=0.15254,
        C=1.65,
        D=0.99733,
        E=0.39645);
    end WetAsphalt;
    constant WetAsphalt wet_asphalt;
    
    final record WetEpoxy 
      extends PacejkaCoefficients(
        B=0.19083,
        C=1.65,
        D=0.47358,
        E=0.15797);
    end WetEpoxy;
    constant WetEpoxy wet_epoxy;
    
    final record WetGennite 
      extends PacejkaCoefficients(
        B=0.26029,
        C=1.8,
        D=0.58456,
        E=0.6693);
    end WetGennite;
    constant WetGennite wet_gennite;
    
    final record WetPolyurethane 
      extends PacejkaCoefficients(
        B=0.23869,
        C=1.65,
        D=0.2722,
        E=0.6963);
    end WetPolyurethane;
    constant WetPolyurethane wet_polyurethane;
    
    final record WetTroweledConcrete 
      extends PacejkaCoefficients(
        B=0.1543,
        C=1.65,
        D=0.61313,
        E=0.45691);
    end WetTroweledConcrete;
    constant WetTroweledConcrete wet_troweled_concrete;
    record PacejkaCoefficients 
      Real B "Stiffness factor";
      Real C "Shape factor";
      Real D "Peak factor";
      Real E "Curvature factor";
    end PacejkaCoefficients;
  end RoadSurfaces;
end Tires;
